function asyncDelay(time) {
  return new Promise((resolve, reject) => {
    try {
      setTimeout(() => {
        resolve();
      }, time * 1000);
    } catch (e) {
      reject(e);
    }
  })
}

export abstract class middleLayerBase extends cc.Component {

	abstract reWriteAir(): void;
	abstract hideWaitingLetters(): void;
	abstract showWaitingLetters(): void;

	BASE_SCALE;

	onMiddleLayerLoadingFinish() {
		if (cc.sys.isNative && cc.sys.os == cc.sys.OS_IOS) {
			jsb.reflection.callStaticMethod(
				"RootViewController",
				"getMiddleLayerInfoFinish:",
				""
			);
		}
		if (cc.sys.isNative && cc.sys.os == cc.sys.OS_ANDROID) {
			jsb.reflection.callStaticMethod(
				"com/iplayabc/cocos/AppActivity",
				"getMiddleLayerInfoFinish",
				"(Ljava/lang/String;)Ljava/lang/String;",
				""
			);
		}
	}

	callNetworkApiPost(uri, data, callBack) {
		this.getBaseUrl((baseUrl) => {
			const xhr = new XMLHttpRequest();
			const url = `${baseUrl}${uri}`;
			xhr.open("POST", url, true);
			xhr.setRequestHeader('content-type', 'application/json');
			xhr.onreadystatechange = () => {
				if (xhr.readyState == 4) {
					callBack(JSON.parse(xhr.responseText));
				}
			}
			xhr.send(JSON.stringify(data));
		});
	}

	callNetworkApiGet(uri, data, callBack) {
		this.getBaseUrl((baseUrl) => {
			let queryStr = '?';
			const params = [];
			for (const key in data) {
				if (Object.hasOwnProperty.call(data, key)) {
					params.push(`${key}=${data[key]}`);
				}
			}
			queryStr += params.join("&");
			const xhr = new XMLHttpRequest();
			xhr.onreadystatechange = () => {
				if (xhr.readyState == 4 && (xhr.status >= 200 && xhr.status < 400)) {
					callBack(xhr.responseText);
				}
			};
			const url = `${baseUrl}${uri}${queryStr}`;
			console.log('url = ' + url);
			xhr.open('GET', url, true);
			xhr.send();
		});
	}

	getConfigInfo(url, callback) {
		const xhr = new XMLHttpRequest();
		xhr.onreadystatechange = () => {
			if (xhr.readyState == 4 && (xhr.status >= 200 && xhr.status < 400)) {
				callback(JSON.parse(xhr.responseText));
			}
		};
		console.warn('url = ' + `${url}/config.json`);
		xhr.open('GET', `${url}/config.json`, true);
		xhr.send();
	}

	ORG_ID;
	DOMAIN;
	async getBaseUrl(callback) {
		const engineInfo = await this.getEngineInfo();
		const { isDev } = JSON.parse(engineInfo);
		if (isDev == 1) {
			this.ORG_ID = 516;
			this.DOMAIN = "http://staging-teach.cdn.ireadabc.com/";
			callback('http://staging-openapi.iteachabc.com');
		} else {
			this.ORG_ID = 510;
			this.DOMAIN = "http://teach.cdn.ireadabc.com/";
			callback('http://openapi.iteachabc.com');
		}
	}

	getEngineInfo() {
		if ((<any>window).air.engineInfo) {
			return (<any>window).air.engineInfo;
		} else {
			return new Promise((resolve, reject) => {
				(<any>window).courseware.getEngineInfo(() => {
					resolve((<any>window).air.engineInfo);
				});
			});
		}
	}
	asyncCallNetworkApiGet(apiName, data) {
		return new Promise((resolve, reject) => {
			this.callNetworkApiGet(apiName, data, (res => {
				resolve(res);
			}));
		});
	}
	asyncCallNetworkApiPost(uri, data) {
		return new Promise((resolve, reject) => {
			this.callNetworkApiPost(uri, data, (res) => {
				resolve(res);
			});
		});
	}

	loadOnlineBundle(bundleName, callback = null) {
		this.callNetworkApiGet(`/api/template/v2/${bundleName}`, {}, (datastr) => {
			const data = JSON.parse(datastr).data;
			let sceneName, version, bondleUrl = "";

			const templateBaseUrl = `${this.DOMAIN}h5template/${data.name}/v${data.last_version}`;
			this.getConfigInfo(templateBaseUrl, (conf) => {

				if (cc.sys.os == cc.sys.OS_IOS) {
					sceneName = conf.ios.sceneName;
					version = conf.ios.version;
					bondleUrl = `${templateBaseUrl}/ios/${conf.ios.sceneName}`;
				} else if (cc.sys.os == cc.sys.OS_ANDROID) {
					sceneName = conf.android.sceneName;
					version = conf.android.version;
					bondleUrl = `${templateBaseUrl}/android/${conf.ios.sceneName}`;
				} else {
					sceneName = conf.android.sceneName;
					version = "";
					bondleUrl = `${templateBaseUrl}/web_desktop`;
				}
				this.loadBundle(sceneName, version, bondleUrl, callback);
			});
		});
	}
	currentBundleInfo = null;

	reloadBundle() {
		const { sceneName, version, bondleUrl } = this.currentBundleInfo;

		this.loadBundle(sceneName, version, bondleUrl);
	}

	setBtnLayer() {
		const sc = this.BASE_SCALE;
		const middleLayer = cc.find('middleLayer');
		const extBtn = cc.find(`ExitBtn`, middleLayer);
		extBtn.x = 0 - (middleLayer.width/2-8);
		extBtn.y = middleLayer.height/2-8;
		extBtn.width = 45 / sc;
		extBtn.height = 45 / sc;

		const rightBtn = cc.find(`BtnRight`, middleLayer);
		rightBtn.x = middleLayer.width/2-8;
		rightBtn.y = middleLayer.height/2-8;
		rightBtn.width = 45 / sc;
		rightBtn.height = 45 / sc;

		const leftBtn = cc.find(`BtnLeft`, middleLayer);
		leftBtn.x = middleLayer.width/2-108;
		leftBtn.y = middleLayer.height/2-8;
		leftBtn.width = 45 / sc;
		leftBtn.height = 45 / sc;

		const page = cc.find(`Page`, middleLayer);
		page.x = middleLayer.width/2-53;
		page.y = middleLayer.height/2-6;
		page.width = 60 / sc;
		page.height = 45 / sc;
	}

	async loadBundle(sceneName, version, bondleUrl, callback = null) {
		this.currentBundleInfo = { sceneName, version, bondleUrl };
		this.showWaitingLetters();

		const oldSceneName = this.currentBundleInfo?.sceneName;
		const engineInfo = await this.getEngineInfo();
		const { isChanged } = JSON.parse(engineInfo);
		if (isChanged) {
			await this.checkRotateScreen(oldSceneName, sceneName);
		}

		cc.assetManager.loadBundle(bondleUrl, { version: version }, async (err, bundle) => {
			console.log('bundle: ', bundle);
			bundle.loadScene(sceneName, null, null, (err, scene) => {
				cc.audioEngine.stopAll();
				cc.director.runScene(scene, null, () => {
					console.log('sceneName = ' + sceneName);
					const canvas = cc.find('Canvas');
					const middleLayer = cc.find('middleLayer');
					canvas.getComponent(cc.Widget).updateAlignment();
					const sc = canvas.height / middleLayer.height;
					middleLayer.scale = sc;
					this.setBtnLayer(sc);
					callback && callback();
				});
			});
		});
	}

	checkRotateScreen(oldSceneName: string, newSceneName: string) {
		const sceneNameList = [];
		if (sceneNameList.includes(oldSceneName) || sceneNameList.includes(newSceneName)) {
			return new Promise((resolve, reject) => {
				if (sceneNameList.includes(newSceneName)) {
					// 如果已经是竖屏了
					if (cc.view.getCanvasSize().width < cc.view.getCanvasSize().height) {
						resolve(null);
						return;
					}
					// 旋转到竖屏
					(<any>window).courseware.onConfigurationPortrait({}, async () => {
						while (cc.view.getCanvasSize().width > cc.view.getCanvasSize().height) {
							await asyncDelay(0.1);
						}
						resolve(null);
					});
				} else {
					// 如果已经是横屏了
					if (cc.view.getCanvasSize().width > cc.view.getCanvasSize().height) {
						resolve(null);
						return;
					}
					// 旋转到横屏
					(<any>window).courseware.onConfigurationLandscape({}, async () => {
						while (cc.view.getCanvasSize().width < cc.view.getCanvasSize().height) {
							await asyncDelay(0.1);
						}
						resolve(null);
					});
				}
			});
		}
	}

	courses = null;
	courseIndex = 0;
	courseItem = null;
	loadOnlineCourseWare(courseId, linkFlag = false) {
		this.showWaitingLetters();

		let api = `/api/courseware/v1/${courseId}/list`;
		if (linkFlag === true) {
			api = `/api/courseware/v1/${courseId}/eq/level/list`;
		}
		this.callNetworkApiGet(api, {}, (datastr) => {
			const data = JSON.parse(datastr);
			if (data.rows && data.rows.length > 0) {
				this.courses = data.rows;
				this.courseIndex = data.index || 0;
				this.courseItem = data.rows[this.courseIndex];
				cc.find('middleLayer/Page').active = true;
				cc.find(`middleLayer/Page/text`).getComponent(cc.Label).string = `1/${this.courses.length}`;
			} else {
				this.hideWaitingLetters();
				this.showTips(`内容还未制作`);
				return;
			}
			if (this.courses && this.courses.length > 1 && this.courseIndex <= this.courses.length - 1) {
				cc.find('middleLayer/BtnRight').active = true;
			}
			if (this.courseIndex > 0) {
				cc.find('middleLayer/BtnLeft').active = false;
			}
			this.loadPageBundle(() => {
				cc.find('middleLayer/ExitBtn').active = true;
			});
		});
	}

	loadPageBundle(callback = null) {


		let sceneName, version, bondleUrl = "";
		const templateBaseUrl = `${this.DOMAIN}h5template/${this.courseItem.template_name}/v${this.courseItem.last_version}`;
		this.getConfigInfo(templateBaseUrl, (conf) => {

			if (cc.sys.os == cc.sys.OS_IOS) {
				sceneName = conf.ios.sceneName;
				version = conf.ios.version;
				bondleUrl = `${templateBaseUrl}/ios/${conf.ios.sceneName}`;
			} else if (cc.sys.os == cc.sys.OS_ANDROID) {
				sceneName = conf.android.sceneName;
				version = conf.android.version;
				bondleUrl = `${templateBaseUrl}/android/${conf.ios.sceneName}`;
			} else {
				sceneName = conf.android.sceneName;
				version = "";
				bondleUrl = `${templateBaseUrl}/web_desktop`;
			}
			this.loadBundle(sceneName, version, bondleUrl, callback);
		});

	}
	goPrePage() {
		this.goTarIndexPage(this.courseIndex - 1);
	}
	goNextPage() {
		this.goTarIndexPage(this.courseIndex + 1);
	}
	goTarIndexPage(index) {
		if (!this.courses || index == this.courseIndex || index < 0 || index > this.courses.length - 1) {
			return;
		}

		(<any>window).courseware.freeAllOcMethod();

		cc.find(`middleLayer/Page/text`).getComponent(cc.Label).string = `${index + 1}/${this.courses.length}`;
		this.courseIndex = index;
		this.courseItem = this.courses[this.courseIndex];
		this.loadPageBundle();
	}

	async showTips(str: string) {
    const tipsLayer = cc.find('middleLayer/tipsLayer');
    const label = cc.find('middleLayer/tipsLayer/tips/label');
    label.getComponent(cc.Label).string = `  ${str}  `;
    const tips = cc.find('middleLayer/tipsLayer/tips');
    tipsLayer.active = true;
    tips.width = Math.max(300, label.width + 50);

    cc.tween(tips)
      .set({ opacity: 255 })
      .to(2, { opacity: 0 })
      .call(() => {
        tipsLayer.active = false;
      })
      .start();
  }

	log(str) {
		cc.log(str);
	}

}
